
/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2011 - DIGITEO - Michael Baudin
* Copyright (C) 2008 - INRIA
* Copyright (C) 2012 - Prateek Papriwal
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#ifndef _CDFLIB_H_
#define _CDFLIB_H_

#ifdef _MSC_VER
#if LIBCDFLIB_EXPORTS 
#define CDFLIB_IMPORTEXPORT __declspec (dllexport)
#else
#define CDFLIB_IMPORTEXPORT __declspec (dllimport)
#endif
#else
#define CDFLIB_IMPORTEXPORT
#endif

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS

//////////////////////////////////////////////////////
//
// All the function below have the same behavior.
// They return a integer status which says if the computation 
// performed well : 
//  * returns CDFLIB_OK if the function can be computed.
//  * returns CDFLIB_ERROR in case of error.
//
// This is the status of a function which performs correctly.
static int CDFLIB_OK = 0;

// The state of a function which generates an error.
static int CDFLIB_ERROR = 1;

//////////////////////////////////////////////////////
// To compute P(X<=x)
static int CDFLIB_LOWERTAIL = 1;

// To compute P(X>x)
static int CDFLIB_UPPERTAIL = 0;

//////////////////////////////////////////////////////
// To compute log(f(x))
static int CDFLIB_LOGCOMPUTE = 1;

// To compute f(x)
static int CDFLIB_LOGNOT = 0;
////////////////////////////////////

// Configure the function which prints messages.
// This function must be called before calling any 
// function.
void cdflib_messageSetFunction ( void (* f)(char * message) );


// Drivers for CDF and quantile functions
int cdflib_cdfnbn(int which, double *p, double *q, double *x, double n, double pr, double ompr);
int cdflib_cdffnc(int which, double *p, double *q, double *x, double dfn, double dfd, double phonc);
int cdflib_cdfchn(int which, double *p, double *q, double *x, double df, double pnonc);


/////////////////////////////////////////////////////////////////////////
//
// Hypergeometric Distribution
//

// Computes the Hypergeometric PDF
//
// Input:
//   x : the number of successful draws in the experiment. x belongs to the set [0,min(k,N)]
//   M : the total size of the population. M belongs to the set {0,1,2,3........}
//   k : the number of successful states in the population. k belongs to the set {0,1,2,3,.......M-1,M}
//   N : the total number of draws in the experiment. N belongs to the set {0,1,2,3.......M-1,M}
// Output:
//   p, output : the probability P(X=x).
//
// Description
//   The function definition is:
//
//f(x,M,k,N) = \frac{\binom{k}{x} \binom{M-k}{N-x}}{\binom{M}{N}}
//
// with the convention that $\binom{M-k}{N-x}$ is zero if <literal>N-x > M-k</literal>.
//
// From the definition of the PDF, the input arguments must be so that:
// x<=N
// x<=k
// k<=M
// N<=M
//
// If one of these conditions is not satisfied, 
// an error is generated.
//
// The implementation uses the fact that:
//
// f(x)=binomialpdf(x,k,p)*binomialpdf(N-x,M-k,p)/binomialpdf(N,M,p)
//
// with p=N/M.
//
// Examples
// int status;
// double y;
// status = cdflib_hygepdf(20,80,50,30,&y);
// expected = 0.1596136;
//
// References
// http://svn.r-project.org/R/trunk/src/nmath/dhyper.c
int cdflib_hygepdf(double x, double M, double k, double N, double *p);
int cdflib_hygeCheckX(char * fname, double x, double M, double k, double N);
int cdflib_hygeCheckParams(char * fname, double M, double k, double N);

// Computes the hypergeometric CDF
//
// Input:
//   lowertail : the tail. 
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//
// Examples
// int status;
// double y;
// status = cdflib_hygecdf(20,80,50,30,1,&y);
// expected = 0.7974774;
int cdflib_hygecdf(double x, double M, double k, double N, int lowertail, double *p);

// Computes the inverse hypergeometric CDF
//
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//   lowertail : the tail. 
// Output:
//   x : the number of successful draws in the experiment. x belongs to the set [0,min(k,N)]
//
// Examples
// status = cdflib_hygeinvcdf(0.2,80,50,30,1,&y)
// expected = 17
int cdflib_hygeinvcdf(double p, double M, double k, double N, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Gamma Distribution
//

// Gamma PDF
//
// Parameters
// Input:
//   x : the outcome, x>=0
//   a : the shape parameter, a>0.
//   b : the scale parameter, b>0.
// Output:
//   y : the density
//
// Description
//   Computes the Gamma probability distribution function.
//
//   The function definition is:
//
//f(x,a,b) = \frac{1}{b^a\Gamma(a)} x^{a-1} \exp\left(-\frac{x}{b}\right)
//
// Compatibility note. 
// 
// This function is compatible with Matlab, but 
// not compatible with R. 
// Indeed, notice that b, the scale, is the inverse of the rate. 
// Other computing languages (including R), use 1/b as the 
// second parameter of the Gamma distribution.
//
// Examples
/* 
int status;
double y;
double expected;
status = distfun_gampdf(1,1,5,&y);
expected = 1.637461506155963586D-01;
*/
int cdflib_gammapdf( double x , double a , double b , double * y );

// Gamma CDF
//
// Parameters
// Input:
//   x : the outcome
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/*
int status;
double p;
double expected;
status = cdflib_gamcdf(1,2,3,CDFLIB_LOWERTAIL,&p);
expected = 0.0446249192349476506148;
*/
int cdflib_gamcdf(double x, double a, double b, int lowertail, double *p );

// Gamma Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome
//
// Examples
/*
int status;
double x;
double expected;
status = cdflib_gaminv(0.5,2,3,CDFLIB_LOWERTAIL,&x);
expected = 5.0350409700499820786490;
*/
int cdflib_gaminv(double p, double a, double b, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// F Distribution
//

// F PDF
//   
// Parameters
// Input:
//   x : x is real and x>=0.
//   v1 : numerator degrees of freendom, v1>0. 
//   v2 : denominator degrees of freendom, v2>0.
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the f distribution function.
//   
//   Any scalar input argument is expanded to a matrix of doubles 
//   of the same size as the other input arguments.
//
// The F distribution has density
//
// f(x) = \frac{\sqrt{\frac{(v_1 x)^{v_1} v_2^{v_2}}{(v_1 x+v_2)^{v_1+v_2}}}}{x B\left(\frac{v_1}{2},\frac{v_2}{2}\right)}
//
// Examples
/*
int status;
double y;
double expected;
status = cdflib_fpdf(3,4,5,&y);
expected = 0.0681795538575894199962;
*/
//
// Bibliography
// Catherine Loader, http://svn.r-project.org/R/trunk/src/nmath/df.c 
//
int cdflib_fpdf(double x,double v1,double v2, double * y);

// F CDF
//   
// Parameters
// Input:
//   x : x is real and x>=0.
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_fcdf(3,4,5,CDFLIB_LOWERTAIL,&p);
expected = 0.8702965153994906000179;
*/
int cdflib_fcdf(double x, double v1, double v2, int lowertail, double *p);

// F Inverse CDF
//   
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : x is real and x>=0.
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_finv(0.7,4,5,CDFLIB_LOWERTAIL,&x);
expected = 1.6286329978911637805794;
*/
int cdflib_finv(double p, double v1, double v2, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Binomial Distribution
//

// Binomial PDF
//
// Parameters
// Input:
//   x : the number of Bernoulli trials after in which success occurs . x belongs to the set {0,1,2,3,...,N}. 
//       x has integer value.
//   n : the total number of binomial trials . N belongs to the set {1,2,3,4,.......}
//       n has integer value.
//   pr : the probability of success in a Bernoulli trial 
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the Binomial distribution function.
//   
//   Any scalar input argument is expanded to a matrix of doubles 
//   of the same size as the other input arguments.
//
//    The function definition is:
//
//f(x,n,pr) = \binom{n}{x} p_r^x (1-p_r)^{n-x}
//
//Examples
/* 
int status;
double y;
double expected;
status = cdflib_binopdf(3,200,0.02,&y);
expected = 0.1963468373887976292647;
*/
int cdflib_binopdf(double x, double n, double p, double * y);

// Binomial CDF
//
// Parameters
// Input:
//   x : the number of Bernoulli trials after in which success occurs . x belongs to the set {0,1,2,3,...,N}. 
//       x has integer value.
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
//Examples
/* 
int status;
double p;
double expected;
status = cdflib_binocdf(3,200,0.02,CDFLIB_LOWERTAIL,&p);
expected = 0.4314949731615046468747;
*/
int cdflib_binocdf(double x, double n, double pr, int lowertail, double *p);

// Binomial Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the number of Bernoulli trials after in which success occurs . x belongs to the set {0,1,2,3,...,N}. 
//       x has integer value.
//
//Examples
/* 
int status;
double x;
double expected;
status = cdflib_binoinv(0.9,200,0.02,CDFLIB_LOWERTAIL,&x);
expected = 7.;
*/
int cdflib_binoinv(double p, double n, double pr, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Normal Distribution
//

// Normal PDF
//
// Parameters
// Input:
//   x : the outcome
//   mu : the mean 
//   sigma : the standard deviation 
// Output:
//   y : the density
//
// Description
//   Computes the probability distribution function 
// of the Normal (Laplace-Gauss) function.
//
//   The function definition is:
//
//f(x,\mu,\sigma) = \frac{1}{\sigma\sqrt{2\pi}} \exp\left(\frac{-(x-\mu)^2}{2\sigma^2}\right)
//
// Examples
/* 
int status;
double y;
double expected;
status = cdflib_normpdf ( 2 , 1.0 , 2.0, &y );
expected = 0.176032663382150;
*/
int cdflib_normpdf(double x, double mu, double sigma, double *y);

// Normal CDF
//
// Parameters
// Input:
//   x : the outcome
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_normcdf ( 2 , 1.0 , 2.0, CDFLIB_LOWERTAIL, &p );
expected = 0.6914624612740130071842;
*/
int cdflib_normcdf(double x, double mu, double sigma, int lowertail, double *p);

// Normal Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_norminv ( 0.7 , 1.0 , 2.0, CDFLIB_LOWERTAIL, &x );
expected = 2.0488010254160813339297;
*/
int cdflib_norminv(double p, double mu, double sigma, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Uniform Distribution
//

// Computes the Uniform PDF
//
// Parameters
// Input:
//   x : the outcome
//   a : the lower bound
//   b : the upper bound
// Output:
//   y : the density.
//
// Description
//   Computes the probability distribution function of 
//   the Uniform distribution function.
//   
//   Any scalar input argument is expanded to a matrix of doubles 
//   of the same size as the other input arguments.
//
//   The function definition is:
//
//f(x,a,b) = 1/(b-a), if x\in[a,b],
//           0,       otherwise.
//
// Examples
// int status;
// double y;
//  status = cdflib_unifpdf ( 4, 3, 6, &y );
//  yexpected = 0.33333;
//
int cdflib_unifpdf(double x, double a, double b, double *y);
int cdflib_unifCheckX(char * fname , double x);
int cdflib_unifCheckParams(char * fname , double a, double b);

// Computes the Uniform CDF
//
// Input:
//   lowertail : the tail. 
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//
// Examples
// int status;
// double y;
//  status = cdflib_unifcdf ( 2, 1, 5, CDFLIB_LOWERTAIL, &y );
//  pexpected = 0.25;
//
int cdflib_unifcdf(double x, double a, double b, int lowertail, double *p);

// Computes the Uniform Inverse CDF
//
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//   lowertail : the tail. 
// Output:
//   x : the outcome.
//
// Examples
// int status;
// double x;
//  status = cdflib_unifinv ( 0.7, 1., 2., CDFLIB_LOWERTAIL, &x );
//  xexpected = 1.7;
//
int cdflib_unifinv(double p, double a, double b, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Chi-Square Distribution
//

// Chi-Square PDF
//   
// Parameters
// Input:
//   x : the outcome, x>=0
//   k : the number of degrees of freedom. k>0 (can be non-integer)
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the Chi-square distribution function.
//
//   The function definition is:
//
//f(x,k) = \frac{1}{2^{\frac{k}{2}}\Gamma\left(\frac{k}{2}\right)} x^{\frac{k}{2}-1} \exp\left(-\frac{x}{2}\right)
//
// Examples
/* 
int status;
double y;
double expected;
status = cdflib_chi2pdf(4,5,&y);
expected = 0.1439759107018347694673;
*/ 
// Bibliography
// http://en.wikipedia.org/wiki/Chi-squared_distribution
int cdflib_chi2pdf(double x, double k, double *y);

// Chi-Square CDF
//   
// Parameters
// Input:
//   x : the outcome, x>=0
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_chi2cdf(4,5,CDFLIB_LOWERTAIL,&p);
expected = 0.4505840486472180161925;
*/ 
int cdflib_chi2cdf(double x, double k, int lowertail, double *p);

// Chi-Square Inverse CDF
//   
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome, x>=0
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_chi2inv(0.7,5,CDFLIB_LOWERTAIL,&x);
expected = 6.0644299841548976459649;
*/ 
int cdflib_chi2inv(double p, double k, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Geometric Distribution
//

// Geometric PDF
//   
// Parameters
// Input:
//   x : the number of Bernoulli trials after which the first success occurs. x belongs to the set {0,1,2,3,.....}
//       x must have an integer value. If not, an error is generated.
//   pr : the probability of success in a Bernoulli trial. pr in [0,1]
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the Geometric distribution function.
//   
//   The function definition is:
//
//f(x,pr) = pr\left(1-pr\right)^x
//
// Compatibility Note : x belongs to the set {0,1,2,3,...}. 
// This choice is compatible with Matlab and R. 
// This is different from Scilab v5 grand(m,n,"geom"), which 
// uses {1,2,3,...}. 
//
// Examples
// int status;
// double y;
// status = cdflib_geopdf(3,0.5,&y);
// yexpected = 0.0625;
//
// Bibliography
// http://en.wikipedia.org/wiki/Geometric_distribution
int cdflib_geopdf(double x, double pr, double *y);
int cdflib_geoCheckParams(char * fname, double pr);
int cdflib_geoCheckX(char * fname, double x);

// Geometric CDF
//
// Parameters
// Input:
//     lowertail: the tail.
// Output:
//     p: the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
// int status;
// double p;
// status = cdflib_geocdf(3,0.5, CDFLIB_LOWERTAIL, &p);
// expected = 0.9375;
//
int cdflib_geocdf(double x, double pr, int lowertail, double *p);

// Geometric InverseCDF
//
// Parameters
// Input:
//     p: the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//     lowertail: the tail.
// Output:
//     x: the outcome
//
// Examples
// int status;
// double x;
// status = cdflib_geoinv(0.999,0.5, CDFLIB_LOWERTAIL, &x);
// xexpected = 9;
//
int cdflib_geoinv(double p, double pr, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// T Distribution
//

// T PDF
//
// Parameters
// Input:
//   x : the outcome.
//   v : the number of degrees of freedom, v>0. 
// Output:
//   y : the density
//
// Description
//   Computes the T probability distribution function.
//
// The function definition is:
//
//f(x,v) = \frac{\Gamma\left(\frac{v+1}{2}\right)}{\Gamma\left(\frac{v}{2}\right)} \frac{1}{\sqrt{v\pi}} \frac{1}{\left(1+\frac{x^2}{v}\right)^{\frac{v+1}{2}}}
//
// Examples
/* 
int status;
double y;
double expected;
status = cdflib_tpdf(-3, 2, &y);
expected = 0.0274101222343421414840;
*/
int cdflib_tpdf(double x, double v, double *y);

// T CDF
//
// Parameters
// Input:
//   x : the outcome.
// Output:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_tcdf(-3, 2, CDFLIB_LOWERTAIL, &p);
expected = 0.0477329831333545562266;
*/
int cdflib_tcdf(double x, double v, int lowertail, double *p);

// T Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome.
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_tinv(0.7, 2, CDFLIB_LOWERTAIL, &x);
expected = 0.6172133998483675387803;
*/
int cdflib_tinv(double p, double v, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Poisson Distribution
//

// Poisson PDF
//   
// Parameters
// Input:
//   x : the number of occurrences of events. x belongs to the set {0,1,2,3,.....}.
//       x must have an integer value.
//   lambda : the average rate of occurrence. We must have lambda>0.
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the poisson distribution function.
//
//   The function definition is:
//
//f(x,\lambda) = \frac{\lambda^x e^{-\lambda}}{x!}
//
// Note : x belongs to the set {0,1,2,3,...}.
//
// Examples
/*
int status;
double y;
double expected;
status = cdflib_poisspdf(0,2,&y);
expected = 0.1353352832366127023178;
*/
// Bibliography
// http://en.wikipedia.org/wiki/Poisson_distribution
int cdflib_poisspdf(double x, double lambda, double *y);
int cdflib_poissCheckX(char * fname, double x);
int cdflib_poissCheckParams(char * fname, double lambda);

int cdfpoi_(int which, double *p, double *q, double *x, double lambda);

// Poisson CDF
//   
// Parameters
// Input:
//   x : the outcome
// Output:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/*
int status;
double p;
double expected;
status = cdflib_poisscdf(1,2,CDFLIB_LOWERTAIL,&p);
expected = 0.4060058497098381069534;
*/
int cdflib_poisscdf(double x, double lambda, int lowertail, double *p);

// Poisson Inverse CDF
//   
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome
//
// Examples
/*
int status;
double x;
double expected;
status = cdflib_poissinv(0.7,2,CDFLIB_LOWERTAIL,&x);
expected = 3.;
*/
int cdflib_poissinv(double p, double lambda, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Log-Normal Distribution
//

// Log-Normal PDF
//
// Parameters
// Input:
//     x: the outcome
//     mu: the mean of the underlying normal variable.
//     sigma: the variance of the underlying normal variable, sigma>0.
// Output:
//     y: the density
//
// Description
//   This function computes the Lognormal PDF.
//
//   Any scalar input argument is expanded to a matrix of doubles of the same size 
//   as the other input arguments.
//
// The Lognormal distribution with parameters mu and sigma has density
//
// f(x) = \frac{1}{\sigma x \sqrt{2 \pi}} \exp\left(-\frac{(\ln(x) - \mu)^2}{2 \sigma^2}\right), 
// if x> 0, and zero otherwise.
//
// Examples
// int status;
// double y;
// status = cdflib_lognpdf ( 2 , 0.0 , 1.0 , &y);
// expected = 0.156874019278981119;
//
// Bibliography
// Dider Pelat, "Bases et mthodes pour le traitement de donnes", section 8.2.8, "Loi log-normale".
// Wikipedia, Lognormal distribution, http://en.wikipedia.org/wiki/Log-normal_distribution
int cdflib_lognpdf(double x, double mu, double sigma, double *y);
int cdflib_lognCheckParams(char * fname, double mu, double sigma);
int cdflib_lognCheckX(char * fname, double x);

// Log-Normal CDF
//
// Parameters
// Input:
//     x: the outcome
// Output:
//     p: the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
// int status;
// double p;
// status = cdflib_logncdf(2., 3., 10., CDFLIB_LOWERTAIL, &p);
// expected = 0.4087797;
int cdflib_logncdf(double x, double mu, double sigma, int lowertail, double *p);

// Log-Normal Inverse CDF
//
// Parameters
// Input:
//     p: the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//     x: the outcome
//
// Examples
// int status;
// double x;
// status = cdflib_logninv(0.6, 1., 2., CDFLIB_LOWERTAIL, &x);
// expected = 4.5117910634839439865;
int cdflib_logninv(double p, double mu, double sigma, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Exponential Distribution
//

// Exponential PDF
//
// Parameters
// Input:
//     x: the outcome
//     mu : the average. mu>0
// Output:
//     y: the density
//
// Description
//   This function computes the Exponential PDF.
//
//   Any scalar input argument is expanded to a matrix of doubles of the same size 
//   as the other input arguments.
//
// Any optional input argument equal to the empty matrix will be set to its
// default value.
//
// The exponential distribution with average mu has density
//
// f(x) = \frac{1}{\mu} e^{\frac{-x}{\mu}}
//
// for x >= 0 and is zero if x<0.
//
// Compatibility note. 
// 
// Notice that mu, the average, is the inverse of the rate. 
// Other computing languages (including R), use 1/mu as the 
// parameter of the exponential distribution.
//
// Examples
// double y;
// int status;
// status=cdflib_exppdf(3,2,&y);
// expected=0.111565080074214909;
//
// Bibliography
// Wikipedia, Exponential distribution function, http://en.wikipedia.org/wiki/Exponential_distribution
int cdflib_exppdf(double x, double mu, double *y);
int cdflib_expCheckX(char * fname, double x);
int cdflib_expCheckParams(char * fname, double mu);

// Exponential CDF
//
// Parameters
// Input:
//     x: the outcome
// Output:
//     p: the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
// double p;
// int status;
// status=cdflib_expcdf(2.,1./3.,CDFLIB_LOWERTAIL,&p);
// expected=9.975212478233337e-1;
//
int cdflib_expcdf(double x, double mu, int lowertail, double *p);

// Exponential Inverse CDF
//
// Parameters
// Input:
//     p: the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//     x: the outcome
//
// Examples
// double x;
// int status;
// status=cdflib_expinv(0.7,2.,CDFLIB_LOWERTAIL,&x);
// expected=2.4079456;
//
int cdflib_expinv(double p, double mu, int lowertail, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Beta Distribution
//

// Beta PDF
//
// Parameters
// Input:
//   x : the outcome. Should be in the [0,1] interval.
//   a : the first shape parameter. a > 0
//   b : the second shape parameter. b > 0
// Output:
//   y : the density
//
// Description
//   Computes the Beta probability distribution function.
//
// Let us denote by B the Beta function. The function definition is:
//
//f(x,a,b) = \frac{1}{B(a,b)} x^{a-1} (1-x)^{b-1}
//
// if x is in [0,1]. 
//
// Examples
/*
int status;
double y;
double expected;
status = cdflib_betapdf ( 0.1 , 2 , 3 , &y);
expected = 0.9720;
*/ 
// Bibliography
// Catherine Loader, http://svn.r-project.org/R/trunk/src/nmath/dbeta.c
//
int cdflib_betapdf(double x, double a, double b, double *y);

// Beta CDF
//
// Parameters
// Input:
//   x : the outcome, x in [0,1]
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/*
int status;
double p;
double expected;
status = cdflib_betacdf( 0.1, 2, 3, CDFLIB_LOWERTAIL, &p);
expected = 0.0523;
 */
int cdflib_betacdf(double x, double a, double b, int lowertail, double *p);

// Beta Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome. 
//
// Examples
/*
int status;
double x;
double expected;
status = cdflib_betainv ( 0.5, 2, 3, CDFLIB_LOWERTAIL, &x);
expected = 0.3857275681323895644148;
 */
int cdflib_betainv(double p, double a, double b, int lowertail, double *x);


__END_DECLS


#endif /** _CDFLIB_H_   **/




