/*
 * Copyright (C) 2012 - Michael Baudin
 * Copyright (C) 2011 - DIGITEO - Michael Baudin
 * Copyright (C) 2008 - INRIA
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */
 
#ifndef _CDFLIB_H_
#define _CDFLIB_H_

#ifdef _MSC_VER
	#if LIBCDFLIB_EXPORTS 
		#define CDFLIB_IMPORTEXPORT __declspec (dllexport)
	#else
		#define CDFLIB_IMPORTEXPORT __declspec (dllimport)
	#endif
#else
	#define CDFLIB_IMPORTEXPORT
#endif

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS

// Configure the function which generates random numbers uniform in [0,1).
// This function must be called at least once.
void genrand_randSetFunction ( double (* f)(void) );

// Configure the function which generates random integers uniform in [a,b].
// This function must be called before at least once.
void genrand_randIntegerInRangeSetFunction ( double (* f)(double *a, double *b) );

// Configure the function which prints messages.
// This function must be called before at least once.
void genrand_messageSetFunction ( void (* f)(char * message) );

// Generate Beta Random Variables
/*
Function
Returns aa single random deviate from the beta distribution with
parameters A and B.  The density of the beta is
x^(aa-1) * (1-x)^(bb-1) / B(aa,bb) for 0 < x < 1
Arguments
A --> First parameter of the beta distribution (A > 0)
B --> Second parameter of the beta distribution (B > 0)
Method
R. C. H. Cheng
Generating Beta Variatew with Nonintegral Shape Parameters
Communications of the ACM, 21:317-322  (1978)
(Algorithms BB and BC)
*/
double genrand_genbet (double a, double b);

// Generate Chi Random Variables
//
/*                              Arguments */
/*     DF --> Degrees of freedom of the chisquare */
/*            (Must be positive) */
/*                              Method */
/*     Uses relation between chisquare and gamma. */
double genrand_genchi (double df);

// Generate Exponential Random Variables
//
/*                              Arguments */
/*     AV --> The mean of the exponential distribution from which */
/*            a random deviate is to be generated. (AV >= 0) */
/*                              Method */
/*     For details see: */
/*               Ahrens, J.H. and Dieter, U. */
/*               Computer Methods for Sampling From the */
/*               Exponential and Normal Distributions. */
/*               Comm. ACM, 15,10 (Oct. 1972), 873 - 882. */
double genrand_genexp (double av);

// Generate F Random Variables
//
/*                              Function */
/*     Generates a random deviate from the F (variance ratio) */
/*     distribution with DFN degrees of freedom in the numerator */
/*     and DFD degrees of freedom in the denominator. */
/*                              Arguments */
/*     DFN --> Numerator degrees of freedom */
/*             (Must be positive) */
/*      DFD --> Denominator degrees of freedom */
/*             (Must be positive) */
/*                              Method */
/*     Directly generates ratio of chisquare variates */
double genrand_genf (double dfn, double dfd);

// Generate Gamma Random Variables
/*                              Function */
/*     Generates random deviates from the gamma distribution whose */
/*     density is proportional*/
/*          X**(A-1) * Exp(-X/B) */
/*                              Arguments */
/*     JJV added the argument ranges supported */
/*     A --> Shape parameter of Gamma distribution ( A > 0 ) */
/*     B --> Scale parameter of Gamma distribution ( B > 0 ) */
/*                              Method */
/*     Renames genrand_sgamma from TOMS as slightly modified by BWB to use RANF */
/*     instead of SUNIF. */
/*     For details see: */
/*               (Case A >= 1.0) */
/*               Ahrens, J.H. and Dieter, U. */
/*               Generating Gamma Variates by a */
/*               Modified Rejection Technique. */
/*               Comm. ACM, 25,1 (Jan. 1982), 47 - 54. */
/*     Algorithm GD */
/*     JJV altered the following to reflect genrand_sgamma argument ranges */
/*               (Case 0.0 < A < 1.0) */
/*               Ahrens, J.H. and Dieter, U. */
/*               Computer Methods for Sampling from Gamma, */
/*               Beta, Poisson and Binomial Distributions. */
/*               Computing, 12 (1974), 223-246/ */
/*     Adapted algorithm GS. */
double genrand_gengam (double a, double b);

// Generate Non-Central Chi Random Variables
/*                              Function */
/*     Generates random deviate  from the  distribution  of a  noncentral */
/*     chisquare with DF degrees  of freedom and noncentrality  parameter */
/*     XNONC. */
/*                              Arguments */
/*     DF --> Degrees of freedom of the chisquare (DF >= 1.0) */
/*     XNONC --> Noncentrality parameter of the chisquare (XNONC >= 0.0) */
/*                              Method */
/*     Uses fact that  noncentral chisquare  is  the  sum of a  chisquare */
/*     deviate with DF-1  degrees of freedom plus the  square of a normal */
/*     deviate with mean sqrt(XNONC) and standard deviation 1. */
double genrand_gennch (double df, double xnonc);

// Generate Non-Central F Random Variables
//
/*     Generates a random deviate from the  noncentral F (variance ratio) */
/*     distribution with DFN degrees of freedom in the numerator, and DFD */
/*     degrees of freedom in the denominator, and noncentrality parameter */
/*     XNONC. */
/*                              Arguments */
/*     DFN --> Numerator degrees of freedom */
/*             (DFN >= 1.0) */
/*      DFD --> Denominator degrees of freedom */
/*             (Must be positive) */
/*     XNONC --> Noncentrality parameter */
/*               (Must be nonnegative) */
/*                              Method */
/*     Directly generates ratio of noncentral numerator chisquare variate */
/*     to central denominator chisquare variate. */
double genrand_gennf (double dfn, double dfd, double xnonc);

// Generate Normal Random Variables
/*                              Function */
/*     Generates a single random deviate from a normal distribution */
/*     with mean, AV, and standard deviation, SD. */
/*                              Arguments */
/*     AV --> Mean of the normal distribution. */
/*     SD --> Standard deviation of the normal distribution. (SD >= 0) */
/*                              Method */
/*     Renames genrand_snorm from TOMS as slightly modified by BWB to use RANF */
/*     instead of SUNIF. */
/*     For details see: */
/*               Ahrens, J.H. and Dieter, U. */
/*               Extensions of Forsythe's Method for Random */
/*               Sampling from the Normal Distribution. */
/*               Math. Comput., 27,124 (Oct. 1973), 927 - 937. */
double genrand_gennor (double av, double sd);

// Generate Multivariate Normal Random Variables
/*                              Arguments */
/*     PARM --> Parameters needed to generate multivariate normal */
/*               deviates (MEANV and Cholesky decomposition of */
/*               COVM). Set by a previous call to genrand_setgmn. */
/*               1 : 1                - size of deviate, P */
/*               2 : P + 1            - mean vector */
/*               P+2 : P*(P+3)/2 + 1  - upper half of cholesky */
/*                                       decomposition of cov matrix */
/*               DOUBLE PRECISION PARM(*) */
/*     X    <-- Vector deviate generated. */
/*              DOUBLE PRECISION X(P) */
/*     WORK <--> Scratch array */
/*               DOUBLE PRECISION WORK(P) */
/*                              Method */
/*     1) Generate P independent standard normal deviates - Ei ~ N(0,1) */
/*     2) Using Cholesky decomposition find A s.t. trans(A)*A = COVM */
/*     3) trans(A)E + MEANV ~ N(MEANV,COVM) */
int genrand_genmn (double *parm, double *x, double *work);

// SET Generate Multivariate Normal random deviate 
/*                              Function */
/*      Places P, MEANV, and the Cholesky factoriztion of COVM */
/*      in PARM for genrand_genmn. */
/*                              Arguments */
/*     MEANV --> Mean vector of multivariate normal distribution. */
/*               DOUBLE PRECISION MEANV(P) */
/*     COVM   <--> (Input) Covariance   matrix    of  the  multivariate */
/*                 normal distribution.  This routine uses only the */
/*                 (1:P,1:P) slice of COVM, but needs to know LDCOVM. */
/*                 (Output) Destroyed on output */
/*                 DOUBLE PRECISION COVM(LDCOVM,P) */
/*     LDCOVM --> Leading actual dimension of COVM. */
/*     P     --> Dimension of the normal, or length of MEANV. */
/*     PARM <-- Array of parameters needed to generate multivariate */
/*              normal deviates (P, MEANV and Cholesky decomposition */
/*              of COVM). */
/*              1 : 1                - P */
/*              2 : P + 1            - MEANV */
/*              P+2 : P*(P+3)/2 + 1  - Cholesky decomposition of COVM */
/*              DOUBLE PRECISION PARM(P*(P+3)/2 + 1) */
int genrand_setgmn (double *meanv, double *covm, int *ldcovm, int *p, double *parm, int *ierr);

// Generate Multinomial Random Variables
/*                              Arguments */
/*     N --> Number of events that will be classified into one of */
/*           the categories 1..NCAT */
/*     P --> Vector of probabilities.  P(i) is the probability that */
/*           an event will be classified into category i.  Thus, P(i) */
/*           must be [0,1]. Only the first NCAT-1 P(i) must be defined */
/*           since P(NCAT) is 1.0 minus the sum of the first */
/*           NCAT-1 P(i). */
/*                         DOUBLE PRECISION P(NCAT-1) */
/*     NCAT --> Number of categories.  Length of P and IX. */
/*                         INTEGER NCAT */
/*     IX <-- Observation from multinomial distribution.  All IX(i) */
/*            will be nonnegative and their sum will be N. */
/*                         INTEGER IX(NCAT) */
/*                              Method */
/*     Algorithm from page 559 of */
/*     Devroye, Luc */
/*     Non-Uniform Random Variate Generation.  Springer-Verlag, */
/*     New York, 1986. */
int genrand_genmul (int n, double *p, int ncat, int *ix);

// Generate Random Permutations
/*               GENerate random PeRMutation of array */
/*                              Arguments */
/*     ARRAY <--> On output ARRAY is a random permutation of its */
/*                 value on input */
/*                         DOUBLE PRECISION ARRAY( LARRAY ) */
/*     LARRAY <--> Length of ARRAY */
/*     Modification by Bruno to handle permutation of a double precision */
/*     array (15/11/2001) and to handle the changes in ignuin */
int genrand_genprm (double *array, int larray);

// Generate Binomial Random Variables
/*
Function
Generates a single random deviate from a binomial
distribution whose number of trials is N and whose
probability of an event in each trial is P.
Arguments
n  --> The number of trials in the binomial distribution
from which a random deviate is to be generated.
p  --> The probability of an event in each trial of the
binomial distribution from which a random deviate
is to be generated.
genrand_ignbin <-- A random deviate yielding the number of events
from N independent trials, each of which has
a probability of event P.
Method
This is algorithm BTPE from:
Kachitvichyanukul, V. and Schmeiser, B. W.
Binomial Random Variate Generation.
Communications of the ACM, 31, 2
(February, 1988) 216.
*/
int genrand_ignbin (int n, double pp);

// Generate Negative Binomial Random Variables
/*                              Function */
/*     Generates a single random deviate from a negative binomial */
/*     distribution. */
/*                              Arguments */
/*     N  --> Required number of events. (N > 0) */
/*     P  --> The probability of an event during a Bernoulli trial. (0.0 < P < 1.0) */
/*                              Method */
/*     Algorithm from page 480 of */
/*     Devroye, Luc */
/*     Non-Uniform Random Variate Generation.  Springer-Verlag, */
/*     New York, 1986. */
int genrand_ignnbn (int n, double p);

// Generate Poisson Random Variables
/*
Function
Generates a single random deviate from a Poisson
distribution with mean AV.
Arguments
av --> The mean of the Poisson distribution from which
a random deviate is to be generated.
Method
Renames KPOIS from TOMS as slightly modified by BWB to use RANF
instead of SUNIF.

References
AHRENS, J.H. AND DIETER, U.
COMPUTER GENERATION OF POISSON DEVIATES
FROM MODIFIED NORMAL DISTRIBUTIONS.
ACM TRANS. MATH. SOFTWARE, 8,2 (JUNE 1982), 163 - 179.
(SLIGHTLY MODIFIED VERSION OF THE PROGRAM IN THE ABOVE ARTICLE)
*/
int genrand_ignpoi (double mu);

// Generate Geometric Random Variables
/*
*  PURPOSE
*     generate a random deviate from G(p) : the geometric
*     law. If a r.v. X ~ G(p), X is the number of Bernouilli trials
*     (B(p)) until succes is met. So X take its values in
*
*         {0,1, 2, 3, ...., } 
*
*     and  P(X=i) = p * (1-p)^(i) 
*
*  METHOD 
*     inversion of the cdf leads to :
*
*     (1)   X = floor( log(1-u) / log(1-p) )
*
*     u being a random deviate from U[0,1).
*
*     by taking into account that 1-u follows also U(0,1)) this may be 
*     replaced with X = ceil( log(u) / log(1-p) )-1 or floor(log(u)/log(1-p))
*     which needs less work. But as ranf() provides number in [0,1[ , 0 may be 
*     gotten and these formulae may give then +oo.
* 
*     With ranf() the max number is 1 - 2^(-32). This let us choose a safe min
*     value for p (to avoid a +oo due to log(1-p)) in the following manner :
*
*        the max is gotten for  M = log(2^(-32)) / (-p)
*
*     (for very small |x|, the accurate func genrand_logp1(x):=log(1+x) return simply x)
*
*     and we want  M <= Rmax (near 1.798+308 in ieee 754 double)
*
*     so p >= 32 log(2)/Rmax which is near 1.234e-307 ; Says pmin = 1.3e-307.
*     (anyway the results gotten for such small values of p are certainly 
*      not meaningful...)
*
*  NOTE
*     this function returns a double instead of an int type : this is 
*     to avoid an extra conversion because in scilab it will be a double.
*
*  ASSUMPTION
*     p must be in [pmin,1]  (to do at the calling level).
*
*/
double genrand_igngeom(double p);

// Generate Uniform Random Variables
//
/*               GeNerate Uniform DOUBLE PRECISION between LOW and HIGH */
/*                              Function */
/*     Generates a DOUBLE PRECISION uniformly distributed between LOW and HIGH. */
/*                              Arguments */
/*     LOW --> Low bound (exclusive) on DOUBLE PRECISION value to be generated */
/*     HIGH --> High bound (exclusive) on DOUBLE PRECISION value to be generated */
double genrand_genunf(double low, double high);

__END_DECLS

#endif /** _CDFLIB_H_   **/




